/***************************************************************************
                          gendate.h  -  header file for Genes date class
                             -------------------
    begin                : Tue Feb 22 2000
    copyright            : (C) 2000 by the Genes Development Team
													 Sean Lamb, Project Lead
    email                : slambo@msn.fullfeed.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/

/* A note about date sorting...
 *
 * It would be silly to sort everything based on the native calendar dates since a
 * person can go through life using several different characters.  For an extreme
 * example, consider a person born in France in 1765.  This person gets married
 * after the French Revolution in the Gregorian year 1797, but the date is recorded
 * in the church records using the French Revolutionary calendar year of 4.  Six years
 * later, this person converts to Islam and his death is recorded using the Islamic
 * calendar.
 *
 * If each date object stores a sorting mechanism, it will make it a little easier
 * for the program to display these events in their proper order (Birth, Marriage, Death)
 * both on the screen and in reports, although the actual date that is displayed will be
 * the date in the appropriate calendar.
 *
 * For this sorting method, we use the astronomical Julian Day.  This is _not_ the
 * calendar date translated into the Julian calendar, rather it is a date system that
 * is used by astronomers, where each day is counted individually starting from Julian
 * Day (JD) 0 (which works out to be the Gregorian date 01 January 4713 BC).  Dates before
 * JD 0 are represented as negative integer values.  For simplicity in this library, we
 * do not include the time of day, which to astronomers equally as important as the date.
 * We do a straight translation from calendar date to Julian Day.
 *
 * One important note here is that Julian Day 0 is used in this library as the default
 * value or if any of the calendar values are unknown.
 *
 * A good discussion of Julian Days is at http://www.magnet.ch/serendipity/hermetic/cal_stud/jdn.htm
 */

#ifndef GENDATE_H
#define GENDATE_H


/**
  *@author Sean Lamb
  */

#include <stdlib.h>
#include "calendar.h"
#include "shared.h"

class GenDate
{

public:
	// Possible calendars...
	enum calendarType { GREGORIAN, JULIAN, JEWISH, FRENCHREV, ISLAMIC, INDIAN, CHINESE };

	// constructors and destructors
	GenDate();
  GenDate(int = 0, int = 0, int = 0, int = GREGORIAN);
	GenDate(const GenDate &);
	~GenDate();

	// various set functions...
  void   gd_set_calendar(int calendar_to_use);
  void   gd_set_date(int = 0, int = 0, int = 0, int = GREGORIAN);
	void	 gd_set_date(char *, int = MDY);

	// various get funcitons...
	int			gd_get_calendar(void)	const { return gd_calendar_; }
	int	    gd_get_cyear(void) const	  { return gd_cyear_; 	 }
	int 		gd_get_cmonth(void) const		{ return gd_cmonth_;	 }
  int   	gd_get_cday(void) const   	{ return gd_cday_;  	 }
	long  	gd_get_jday(void) const   	{ return gd_jday_;  	 }
	string	gd_get_date_string(int);

 	// Testing functions
	bool	 gd_isvalid(void);

private:
	int    gd_calendar_;  // The calendar to use for the current object
	long   gd_jday_;      // The Julian Day corresponding to the calendar date entered
	int    gd_cyear_, gd_cmonth_, gd_cday_;  // The year, month and day in the chosen calendar

  // helper functions
  bool   gd_convert_new_calendar(void);

};

#endif


